#+ELEVENTY_COLLECTIONS: _posts
#+ELEVENTY_BASE_DIR: ~/proj/static-blog/
#+ELEVENTY_CATEGORIES+: emacs
#+ELEVENTY_LAYOUT: layouts/post
#+ELEVENTY_BASE_URL: https://sachachua.com
* Using Emacs Lisp to export TXT/EPUB/PDF from Org Mode to the Supernote via Browse and Access :supernote:org:emacs:
SCHEDULED: <2024-09-29 Sun>
:PROPERTIES:
:CREATED:  [2024-09-29 Sun 07:58]
:EXPORT_LATEX_HEADER: \usepackage{setspace}
:EXPORT_DATE: 2024-09-29T08:27:21-0400
:EXPORT_ELEVENTY_PERMALINK: /blog/2024/09/using-emacs-lisp-to-export-txt-epub-pdf-from-org-mode-to-the-supernote-via-browse-and-access/
:EXPORT_ELEVENTY_FILE_NAME: blog/2024/09/using-emacs-lisp-to-export-txt-epub-pdf-from-org-mode-to-the-supernote-via-browse-and-access/
:CUSTOM_ID: supernote-org-upload
:END:

#+begin_export latex
\spacing{1.5}
#+end_export

I've been experimenting with the Supernote's
Browse and Access feature because I want to be
able to upload files quickly instead of waiting
for Dropbox to synchronize. First, I want to store
the IP address in a variable:

[[defvar:my-supernote-ip-address?open=1]]

Here's how to upload:

#+begin_src emacs-lisp :results silent
(defun my-supernote-upload (filename &optional supernote-path)
	(interactive "FFile: ")
	(setq supernote-path (or supernote-path "/INBOX"))
	(let* ((boundary (mml-compute-boundary '()))
				 (url-request-method "POST")
				 (url-request-extra-headers
					`(("Content-Type" . ,(format "multipart/form-data; boundary=%s" boundary))))
				 (url-request-data
					(mm-url-encode-multipart-form-data
					 `(("file" . (("name" . "file")
												("filename" . ,(file-name-nondirectory filename))
												("content-type" . "application/octet-stream")
												("filedata" . ,(with-temp-buffer
																				 (insert-file-contents-literally filename)
																				 (buffer-substring-no-properties (point-min) (point-max)))))))
					 boundary)))
		(with-current-buffer
				(url-retrieve-synchronously
				 (format "http://%s:8089%s" my-supernote-ip-address supernote-path))
			(re-search-backward "^$")
			(prog1 (json-read)
				(kill-buffer)))))
#+end_src

HTML isn't supported. Text works, but it doesn't support annotation. PDF or EPUB could work.
It would make sense to register this as an export backend so that I can call it as part of the usual export process.

#+begin_src emacs-lisp :results silent
(defun my-supernote-org-upload-as-text (&optional async subtree visible-only body-only ext-plist)
	"Export Org format, but save it with a .txt extension."
	(interactive (list nil current-prefix-arg))
	(let ((filename (org-export-output-file-name ".txt" subtree))
				(text (org-export-as 'org subtree visible-only body-only ext-plist)))
		;; consider copying instead of exporting so that #+begin_export html etc. is preserved
		(with-temp-file filename
			(insert text))
		(my-supernote-upload filename)))

(defun my-supernote-org-upload-as-pdf (&optional async subtree visible-only body-only ext-plist)
	(interactive (list nil current-prefix-arg))
	(my-supernote-upload (org-latex-export-to-pdf async subtree visible-only body-only ext-plist)))

(defun my-supernote-org-upload-as-epub (&optional async subtree visible-only body-only ext-plist)
	(interactive (list nil current-prefix-arg))
	(my-supernote-upload (org-epub-export-to-epub async subtree visible-only ext-plist)))

(org-export-define-backend
		'supernote nil
		:menu-entry '(?s "Supernote"
										 ((?s "as PDF" my-supernote-org-upload-as-pdf)
											(?e "as EPUB" my-supernote-org-upload-as-epub)
											(?o "as Org" my-supernote-org-upload-as-text))))
#+end_src

Adding this line to my Org file allows me to use ~\spacing{1.5}~ for 1.5 line spacing, so I can write in more annotations..

#+begin_example
,#+LATEX_HEADER+: \usepackage{setspace}
#+end_example

Sometimes I use custom blocks for HTML classes. When LaTeX complains about undefined environments, I can define them like this:

#+begin_example
,#+LATEX_HEADER+: \newenvironment{whatever_my_custom_environment_is_called}
#+end_example

Now I can export a subtree or file to my Supernote for easy review.

I wonder if multimodal AI models can handle annotated images with editing marks...

